@@ -672,8 +672,6 @@ got_packidx_match_id_str_prefix(struct got_object_id_queue *matched_ids,
 	struct got_packidx_object_id *oid;
 	uint32_t i = 0;
 
-	STAILQ_INIT(matched_ids);
-
 	if (prefix_len < 2)
 		return got_error_path(id_str_prefix, GOT_ERR_BAD_OBJ_ID_STR);
 
@@ -692,7 +690,7 @@ got_packidx_match_id_str_prefix(struct got_object_id_queue *matched_ids,
 		int cmp;
 
 		if (!got_sha1_digest_to_str(oid->sha1, id_str, sizeof(id_str)))
-		        return got_error(GOT_ERR_NO_SPACE);
+			return got_error(GOT_ERR_NO_SPACE);
 
 		cmp = strncmp(id_str, id_str_prefix, prefix_len);
 		if (cmp < 0) {
@@ -703,16 +701,14 @@ got_packidx_match_id_str_prefix(struct got_object_id_queue *matched_ids,
 
 		err = got_object_qid_alloc_partial(&qid);
 		if (err)
-			break;
+			return err;
 		memcpy(qid->id.sha1, oid->sha1, SHA1_DIGEST_LENGTH);
 		STAILQ_INSERT_TAIL(matched_ids, qid, entry);
 
 		oid = &packidx->hdr.sorted_ids[++i];
 	}
 
-	if (err)
-		got_object_id_queue_free(matched_ids);
-	return err;
+	return NULL;
 }
 
 static void
@@ -941,7 +937,7 @@ parse_negative_offset(int64_t *offset, size_t *len, struct got_pack *pack,
 		if (pack->map) {
 			size_t mapoff;
 
-			if (delta_offset + *len > SIZE_MAX) {
+			if (delta_offset > SIZE_MAX - *len) {
 				return got_error_fmt(GOT_ERR_PACK_OFFSET,
 				    "mapoff %lld would overflow size_t",
 				    (long long)delta_offset + *len);
@@ -1099,7 +1095,7 @@ got_pack_parse_ref_delta(struct got_object_id *id,
 	if (pack->map) {
 		size_t mapoff;
 
-		if (delta_offset + tslen > SIZE_MAX) {
+		if (delta_offset > SIZE_MAX - tslen) {
 			return got_error_fmt(GOT_ERR_PACK_OFFSET,
 			    "mapoff %lld would overflow size_t",
 			    (long long)delta_offset + tslen);
@@ -1467,12 +1463,13 @@ got_pack_dump_delta_chain_to_file(size_t *result_size,
 				accum_bufsz = max_size;
 				if (pack->map) {
 					if (delta_data_offset > SIZE_MAX) {
-						return got_error_fmt(
+						err = got_error_fmt(
 						    GOT_ERR_RANGE,
 						    "delta offset %lld "
 						    "overflows size_t",
 						    (long long)
 						    delta_data_offset);
+						goto done;
 					}
 
 					mapoff = delta_data_offset;
@@ -1646,6 +1643,10 @@ got_pack_dump_delta_chain_to_file(size_t *result_size,
 
 done:
 	free(base_buf);
+	if (err) {
+		free(accum_buf);
+		accum_buf = NULL;
+	}
 	if (accum_buf) {
 		size_t len = fwrite(accum_buf, 1, accum_size, outfile);
 		free(accum_buf);
