@@ -141,12 +141,16 @@ got_privsep_recv_imsg(struct imsg *imsg, struct imsgbuf *ibuf,
 			return got_error_from_errno("imsg_get");
 	}
 
-	if (imsg->hdr.len < IMSG_HEADER_SIZE + min_datalen)
-		return got_error(GOT_ERR_PRIVSEP_LEN);
-
 	if (imsg->hdr.type == GOT_IMSG_ERROR) {
 		size_t datalen = imsg->hdr.len - IMSG_HEADER_SIZE;
-		return recv_imsg_error(imsg, datalen);
+		err = recv_imsg_error(imsg, datalen);
+		imsg_free(imsg);
+		return err;
+	}
+
+	if (imsg->hdr.len < IMSG_HEADER_SIZE + min_datalen) {
+		imsg_free(imsg);
+		return got_error(GOT_ERR_PRIVSEP_LEN);
 	}
 
 	return NULL;
@@ -174,8 +178,9 @@ got_privsep_send_error(struct imsgbuf *ibuf, const struct got_error *err)
 
 	poll_err = got_poll_fd(ibuf->fd, POLLOUT, INFTIM);
 	if (poll_err) {
-		fprintf(stderr, "%s: error %d \"%s\": poll: %s\n",
-		    getprogname(), err->code, err->msg, poll_err->msg);
+		if (poll_err->code != GOT_ERR_EOF)
+			fprintf(stderr, "%s: error %d \"%s\": poll: %s\n",
+			    getprogname(), err->code, err->msg, poll_err->msg);
 		return;
 	}
 
@@ -1183,8 +1188,7 @@ got_privsep_recv_obj(struct got_object **obj, struct imsgbuf *ibuf)
 {
 	const struct got_error *err = NULL;
 	struct imsg imsg;
-	const size_t min_datalen =
-	    MIN(sizeof(struct got_imsg_error), sizeof(struct got_imsg_object));
+	const size_t min_datalen = sizeof(struct got_imsg_object);
 
 	*obj = NULL;
 
@@ -1405,9 +1409,7 @@ got_privsep_recv_commit(struct got_commit_object **commit, struct imsgbuf *ibuf)
 	const struct got_error *err = NULL;
 	struct imsg imsg;
 	size_t datalen;
-	const size_t min_datalen =
-	    MIN(sizeof(struct got_imsg_error),
-	    sizeof(struct got_imsg_commit_object));
+	const size_t min_datalen = sizeof(struct got_imsg_commit_object);
 
 	*commit = NULL;
 
@@ -1592,9 +1594,7 @@ const struct got_error *
 got_privsep_recv_tree(struct got_tree_object **tree, struct imsgbuf *ibuf)
 {
 	const struct got_error *err = NULL;
-	const size_t min_datalen =
-	    MIN(sizeof(struct got_imsg_error),
-	    sizeof(struct got_imsg_tree_object));
+	const size_t min_datalen = sizeof(struct got_imsg_tree_object);
 	struct got_imsg_tree_object *itree;
 	int nentries = 0;
 
@@ -1849,9 +1849,7 @@ got_privsep_recv_tag(struct got_tag_object **tag, struct imsgbuf *ibuf)
 	struct imsg imsg;
 	struct got_imsg_tag_object *itag;
 	size_t len, datalen;
-	const size_t min_datalen =
-	    MIN(sizeof(struct got_imsg_error),
-	    sizeof(struct got_imsg_tag_object));
+	const size_t min_datalen = sizeof(struct got_imsg_tag_object);
 
 	*tag = NULL;
 
@@ -2206,8 +2204,7 @@ got_privsep_recv_gitconfig_int(int *val, struct imsgbuf *ibuf)
 	const struct got_error *err = NULL;
 	struct imsg imsg;
 	size_t datalen;
-	const size_t min_datalen =
-	    MIN(sizeof(struct got_imsg_error), sizeof(int));
+	const size_t min_datalen = sizeof(int);
 
 	*val = 0;
 
@@ -2274,8 +2271,8 @@ got_privsep_recv_gitconfig_remotes(struct got_remote_repo **remotes,
 	switch (imsg.hdr.type) {
 	case GOT_IMSG_GITCONFIG_REMOTES:
 		if (datalen != sizeof(iremotes)) {
-			err = got_error(GOT_ERR_PRIVSEP_LEN);
-			break;
+			imsg_free(&imsg);
+			return got_error(GOT_ERR_PRIVSEP_LEN);
 		}
 		memcpy(&iremotes, imsg.data, sizeof(iremotes));
 		if (iremotes.nremotes == 0) {
@@ -2484,8 +2481,7 @@ got_privsep_recv_gotconfig_remotes(struct got_remote_repo **remotes,
 	size_t datalen;
 	struct got_imsg_remotes iremotes;
 	struct got_imsg_remote iremote;
-	const size_t min_datalen =
-	    MIN(sizeof(struct got_imsg_error), sizeof(iremotes));
+	const size_t min_datalen = sizeof(iremotes);
 
 	*remotes = NULL;
 	*nremotes = 0;
@@ -2525,8 +2521,7 @@ got_privsep_recv_gotconfig_remotes(struct got_remote_repo **remotes,
 
 	while (*nremotes < iremotes.nremotes) {
 		struct got_remote_repo *remote;
-		const size_t min_datalen =
-		    MIN(sizeof(struct got_imsg_error), sizeof(iremote));
+		const size_t min_datalen = sizeof(iremote);
 		int i;
 
 		err = got_privsep_recv_imsg(&imsg, ibuf, min_datalen);
@@ -2697,7 +2692,6 @@ got_privsep_send_commit_traversal_request(struct imsgbuf *ibuf,
 
 const struct got_error *
 got_privsep_recv_traversed_commits(struct got_commit_object **changed_commit,
-    struct got_object_id **changed_commit_id,
     struct got_object_id_queue *commit_ids, struct imsgbuf *ibuf)
 {
 	const struct got_error *err = NULL;
@@ -2708,7 +2702,6 @@ got_privsep_recv_traversed_commits(struct got_commit_object **changed_commit,
 	int i, done = 0;
 
 	*changed_commit = NULL;
-	*changed_commit_id = NULL;
 
 	while (!done) {
 		err = got_privsep_recv_imsg(&imsg, ibuf, 0);
@@ -2734,23 +2727,9 @@ got_privsep_recv_traversed_commits(struct got_commit_object **changed_commit,
 				memcpy(&qid->id, &ids[i], sizeof(ids[i]));
 				STAILQ_INSERT_TAIL(commit_ids, qid, entry);
 
-				/* The last commit may contain a change. */
-				if (i == icommits->ncommits - 1) {
-					*changed_commit_id =
-					    got_object_id_dup(&qid->id);
-					if (*changed_commit_id == NULL) {
-						err = got_error_from_errno(
-						    "got_object_id_dup");
-						break;
-					}
-				}
 			}
 			break;
 		case GOT_IMSG_COMMIT:
-			if (*changed_commit_id == NULL) {
-				err = got_error(GOT_ERR_PRIVSEP_MSG);
-				break;
-			}
 			err = get_commit_from_imsg(changed_commit, &imsg,
 			    datalen, ibuf);
 			break;
@@ -2787,7 +2766,7 @@ got_privsep_send_enumerated_tree(size_t *totlen, struct imsgbuf *ibuf,
 	if (wbuf == NULL)
 		return got_error_from_errno("imsg_create ENUMERATED_TREE");
 
-	if (imsg_add(wbuf, tree_id->sha1, SHA1_DIGEST_LENGTH) == -1)
+	if (imsg_add(wbuf, tree_id, sizeof(*tree_id)) == -1)
 		return got_error_from_errno("imsg_add ENUMERATED_TREE");
 	if (imsg_add(wbuf, &nentries, sizeof(nentries)) == -1)
 		return got_error_from_errno("imsg_add ENUMERATED_TREE");
@@ -2845,12 +2824,12 @@ got_privsep_send_enumerated_commit(struct imsgbuf *ibuf,
 	struct ibuf *wbuf;
 
 	wbuf = imsg_create(ibuf, GOT_IMSG_ENUMERATED_COMMIT, 0, 0,
-	    sizeof(struct got_imsg_enumerated_commit) + SHA1_DIGEST_LENGTH);
+	    sizeof(struct got_imsg_enumerated_commit));
 	if (wbuf == NULL)
 		return got_error_from_errno("imsg_create ENUMERATED_COMMIT");
 
 	/* Keep in sync with struct got_imsg_enumerated_commit! */
-	if (imsg_add(wbuf, id, SHA1_DIGEST_LENGTH) == -1)
+	if (imsg_add(wbuf, id, sizeof(*id)) == -1)
 		return got_error_from_errno("imsg_add ENUMERATED_COMMIT");
 	if (imsg_add(wbuf, &mtime, sizeof(mtime)) == -1)
 		return got_error_from_errno("imsg_add ENUMERATED_COMMIT");
@@ -2901,7 +2880,7 @@ got_privsep_recv_enumerated_objects(int *found_all_objects,
 				break;
 			}
 			icommit = (struct got_imsg_enumerated_commit *)imsg.data;
-			memcpy(commit_id.sha1, icommit->id, SHA1_DIGEST_LENGTH);
+			memcpy(&commit_id, &icommit->id, sizeof(commit_id));
 			mtime = icommit->mtime;
 			have_commit = 1;
 			break;
@@ -2921,7 +2900,7 @@ got_privsep_recv_enumerated_objects(int *found_all_objects,
 				err = got_error(GOT_ERR_PRIVSEP_LEN);
 				break;
 			}
-			memcpy(tree_id.sha1, itree->id, sizeof(tree_id.sha1));
+			memcpy(&tree_id, &itree->id, sizeof(tree_id));
 			free(path);
 			path = strndup(imsg.data + sizeof(*itree), path_len);
 			if (path == NULL) {
@@ -3448,7 +3427,7 @@ send_painted_commits(struct got_object_id_queue *ids, int *nids,
 		color = (intptr_t)qid->data;
 
 		/* Keep in sync with struct got_imsg_painted_commit! */
-		if (imsg_add(wbuf, qid->id.sha1, SHA1_DIGEST_LENGTH) == -1)
+		if (imsg_add(wbuf, &qid->id, sizeof(qid->id)) == -1)
 			return got_error_from_errno("imsg_add PAINTED_COMMITS");
 		if (imsg_add(wbuf, &color, sizeof(color)) == -1)
 			return got_error_from_errno("imsg_add PAINTED_COMMITS");
@@ -3510,10 +3489,8 @@ got_privsep_recv_painted_commits(struct got_object_id_queue *new_ids,
 
 	for (;;) {
 		err = got_privsep_recv_imsg(&imsg, ibuf, 0);
-		if (err){
-			imsg_free(&imsg);
+		if (err)
 			return err;
-		}
 
 		datalen = imsg.hdr.len - IMSG_HEADER_SIZE;
 		if (imsg.hdr.type == GOT_IMSG_COMMIT_PAINTING_DONE) {
@@ -3544,7 +3521,7 @@ got_privsep_recv_painted_commits(struct got_object_id_queue *new_ids,
 
 			if (icommits.present_in_pack) {
 				struct got_object_id id;
-				memcpy(id.sha1, icommit.id, SHA1_DIGEST_LENGTH);
+				memcpy(&id, &icommit.id, sizeof(id));
 				err = cb(cb_arg, &id, icommit.color);
 				if (err)
 					break;
@@ -3553,8 +3530,8 @@ got_privsep_recv_painted_commits(struct got_object_id_queue *new_ids,
 				err = got_object_qid_alloc_partial(&qid);
 				if (err)
 					break;
-				memcpy(qid->id.sha1, icommit.id,
-				    SHA1_DIGEST_LENGTH);
+				memcpy(&qid->id, &icommit.id,
+				    sizeof(qid->id));
 				qid->data = (void *)icommit.color;
 				STAILQ_INSERT_TAIL(new_ids, qid, entry);
 			}
@@ -3582,6 +3559,7 @@ got_privsep_unveil_exec_helpers(void)
 	    GOT_PATH_PROG_FETCH_PACK,
 	    GOT_PATH_PROG_INDEX_PACK,
 	    GOT_PATH_PROG_SEND_PACK,
+	    /* GOT_PATH_PROG_FETCH_HTTP explicitly excluded */
 	};
 	size_t i;
 
