@@ -1426,6 +1426,7 @@ got_repo_list_packidx(struct got_pathlist_head *packidx_paths,
 	packdir = fdopendir(packdir_fd);
 	if (packdir == NULL) {
 		err = got_error_from_errno("fdopendir");
+		close(packdir_fd);
 		goto done;
 	}
 
@@ -1806,7 +1807,6 @@ retry:
 				    "modifications");
 				goto done;
 			}
-			got_object_id_queue_free(&matched_ids);
 			goto retry;
 		}
 
@@ -1817,6 +1817,8 @@ retry:
 		if (err)
 			break;
 
+		got_object_id_queue_free(&matched_ids);
+
 		err = got_packidx_match_id_str_prefix(&matched_ids,
 		    packidx, id_str_prefix);
 		if (err) {
@@ -2150,13 +2152,30 @@ got_repo_object_match_tag(struct got_tag_object **tag, const char *name,
 	return err;
 }
 
+const struct got_error *
+got_repo_find_object_id(struct got_object_id *id, struct got_repository *repo)
+{
+	const struct got_error *err;
+	struct got_object_id *matched_id = NULL;
+	char *id_str = NULL;
+
+	err = got_object_id_str(&id_str, id);
+	if (err)
+		return err;
+
+	err = got_repo_match_object_id_prefix(&matched_id, id_str,
+	    GOT_OBJ_TYPE_ANY, repo);
+	free(id_str);
+	return err;
+}
+
 static const struct got_error *
 alloc_added_blob_tree_entry(struct got_tree_entry **new_te,
     const char *name, mode_t mode, struct got_object_id *blob_id)
 {
 	const struct got_error *err = NULL;
 
-	 *new_te = NULL;
+	*new_te = NULL;
 
 	*new_te = calloc(1, sizeof(**new_te));
 	if (*new_te == NULL)
@@ -2525,6 +2544,7 @@ got_repo_get_packfile_info(int *npackfiles, int *nobjects,
 	packdir = fdopendir(packdir_fd);
 	if (packdir == NULL) {
 		err = got_error_from_errno("fdopendir");
+		close(packdir_fd);
 		goto done;
 	}
 
